(function($, owner) {
	owner.versionInfoUrl = commonProtocol.serverDomain + '/appVersionInfo.json?appv=' + app.appVersion + '&t=' + Date.now();
	// 下载版本信息，解析检查
	owner.checkAppVersion = function(plusObj, callback) {
		if (!plusObj) {
			return;
		}
		callback = callback || $.noop;
		downloadTask = plusObj.downloader.createDownload(owner.versionInfoUrl,
			{method: 'GET', retryInterval: 10, filename: '_downloads/' + Date.now() + '.json'}, function(download, statusCode) {
			if (200 != statusCode) {
				callback(false);
				return;
			}
			var filePath = download.filename;
			if (!filePath) {
				callback(false);
				return;
			}
			plusObj.io.resolveLocalFileSystemURL(filePath, function(entry) {
				entry.file(function(file) {
					var fileReader = new plusObj.io.FileReader();
					fileReader.readAsText(file, 'utf-8');
					fileReader.onloadend = function(evt) {
						var data = evt.target.result;
						try {
							data = JSON.parse(data);
						} catch(ex) {
							callback(false);
							return;
						}
						if ($.os.ios) {
							data = data.ios;
						} else if ($.os.android) {
							data = data.android;
						}
						if (!data.addr || !data.vlist) {
							callback(false);
							return;
						}
						var maxVInfo = {id: app.appVersionId, version: '', force: false, address: '', desc: ''};
						for (var i=data.vlist.length-1; i >= 0; --i) {
							var oneVersion = data.vlist[i];
							if (oneVersion.id > app.appVersionId) {
								if (!maxVInfo.force && (1 == oneVersion.force)) {
									maxVInfo.force = true;
								}
								if (oneVersion.id > maxVInfo.id) {
									maxVInfo.version = oneVersion.version;
									maxVInfo.desc = oneVersion.desc;
									maxVInfo.id = oneVersion.id;
								}
								if (!maxVInfo.address) {
									maxVInfo.address = data.addr;
								}
							}
						}
						maxVInfo.needUpdate = maxVInfo.id > app.appVersionId;
						callback(true, maxVInfo);
					}
				});
			}, function(e) {});
		});
		downloadTask.start();
	};
	
	// 下载apk
	owner.downloadApp = function(plusObj, appAddr, callback) {
		callback = callback || $.noop;
		if (!appAddr) {
			callback('下载地址为空');
			return;
		}
		downloadTask = plusObj.downloader.createDownload(appAddr,
			{method: 'GET', retryInterval: 10, filename: '_downloads/' + Date.now() + appAddr.slice(-4)}, function(download, statusCode) {
			if (200 != statusCode) {
				callback('下载失败，错误代码是' + statusCode);
				return;
			}
			var filePath = download.filename;
			if (!filePath) {
				callback('下载失败，本地保存不成功');
				return;
			}
			plusObj.io.resolveLocalFileSystemURL(filePath, function(entry) {
				callback(false, entry.toLocalURL());
			}, function(e) {});
		});
		downloadTask.start();
	};
	// 安装
	owner.installApp = function(plusObj, appFile, force) {
		plusObj.runtime.install(appFile, {force: true}, function(info) {
      		plusObj.runtime.quit();			
      	}, function(ex) {
      	   	plusObj.nativeUI.confirm('安装失败，您可以点击按钮重试', function(e) {
      	   		if (0 == e.index) {
      	   			installApp(plusObj, appFile, force);
      	   		}
      		}, null, force ? ['重试', '取消并退出'] : ['重试', '取消']);
    	});
	}
	//下载并开始安装
	owner.updateApp = function(plusObj, appAddr, force) {
		var waiting = plusObj.nativeUI.showWaiting('正在下载安装包...');
		versionChecker.downloadApp(plusObj, appAddr, function(errMsg, appFile) {
			waiting.close();
      		if (errMsg) {
      			plusObj.nativeUI.confirm('下载失败，您可以点击按钮重试', function(e) {
      	    			if (0 == e.index) {
      	    				owner.updateApp(plusObj, appAddr, force);
      	    			}
      	    		}, null, force ? ['重试', '取消并退出'] : ['重试', '取消']);
    				return;
			}
      	    owner.installApp(plusObj, appFile, force);
     	});
	}
	// 循环弹窗，因为在iOS上无法使用代码退出应用，所以当要强制更新时循环弹出窗口
	owner.alertForever = function(plusObj, address, version) {
		plusObj.nativeUI.confirm('有新的版本' + version + '，现在开始更新？', function(e) {
			if (0 == e.index) {
				plusObj.runtime.openURL(address);
			}
			setTimeout(function() {
				owner.alertForever(plusObj, address, version);
			}, 800);
		}, null, ['确定']);
	};
	// 开始查检
	owner.startCheckUpdate = function(plusObj, showWaiting, noUpdateCallback) {
		noUpdateCallback = noUpdateCallback || $.noop;
		var waiting = null;
		showWaiting && (waiting = plusObj.nativeUI.showWaiting());
		versionChecker.checkAppVersion(plusObj, function(success, data) {
			waiting && waiting.close();
			waiting = false;
      		if (!success) {
      			noUpdateCallback();
      	    		return;
      	    }
      	    if (data.needUpdate) {
      	    		var btns = ['确定'];
      	    		var msg = '有新的版本' + data.version + '，现在开始更新？';
      	    		if (data.force && $.os.android) {
      	    			btns.push('取消并退出');
      	    		} else if (!data.force) {
      	    			btns.push('稍后更新');
      	    		}
      	    		plusObj.nativeUI.confirm(msg, function(e) {
      	    			if (0 == e.index) {
      	    				if ($.os.android) {
      	    					owner.updateApp(plusObj, commonProtocol.serverDomain + data.address, data.force);
      					} else if ($.os.ios) {
      	    					plusObj.runtime.openURL(data.address);
      	    					if (data.force) {
      	    						setTimeout(function() {
      	    							owner.alertForever(plusObj, data.address, data.version);
      	    						}, 800);
      	    					}
      	    				}
      	    			} else if (data.force) {
      	    				if ($.os.android) {
      	    					plusObj.runtime.quit();
      	    				} else if ($.os.ios) {
      	    					owner.alertForever(plusObj, data.address, data.version);
      	    				}
      	    			}
      	   		}, null, btns);
   			} else {
   				noUpdateCallback();
   			}
		});
		
		// 可能在某些低版本的android系统下downloader不回调，
		//	解决办法是60秒后如果没有回调则停止检查
		setTimeout(function() {
			waiting && waiting.close();
			// TODO -- 使用接口检查更新
		}, 60000);
	};
}(mui, window.versionChecker = {}));
