(function($, doc, owner) {
	var DataKey = '$DATA_KEY'
	var DirPath = '_downloads/';
	var plusObj = false;
	var storageObj = false;
	var downloader = false;
	var ioObj = false;
	var metadata = false;
	
	// private function, reset metadate
	function resetMetadata() {
		metadata = {};
		metadata.keyArray = new Array();
		metadata.valueArray = new Array();
	}
	
	// private function, reset metadata from local storage
	function resetMetadataFromLocalStorage() {
		var metadataStr = storageObj.getItem(DataKey);
		if (!metadataStr) {
			resetMetadata();
		} else {
			try {
				metadata = JSON.parse(metadataStr);
				if (!metadata.keyArray) {
					metadata.keyArray = new Array();
				}
				if (!metadata.valueArray) {
					metadata.valueArray = new Array();
				}
			} catch(e) {
				resetMetadata();
			}
		}
	}
	
	// init file manager
	owner.initManager = function(plusInstance) {
		if (!plusInstance) {
			return false;
		}
		plusObj = plusInstance || false;
		storageObj = plusObj.storage;
		downloader = plusObj.downloader;
		ioObj = plusObj.io;
		resetMetadataFromLocalStorage();
		return true;
	};
	
	// private function, check update and plus object
	function checkContext() {
		if (false === plusObj) {
			owner.initManager(plus);
		}
		return true;
	}
	
	// private function, save meta data
	function saveMetadata() {
		if (false === metadata) {
			return;
		}
		var metadataStr = false;
		try {
			metadataStr = JSON.stringify(metadata);
		} catch(e) {
			metadataStr = '{}';
		}
		storageObj.setItem(DataKey, metadataStr);
	}
	
	// clear cache, param of callback: false(fail) or true(success)
	owner.clearCache = function(callback) {
		callback = callback || $.noop;
		if (!checkContext()) {
			callback(false);
			return;
		}
		ioObj.resolveLocalFileSystemURL(DirPath, function(entry) {
			if (!entry || !entry.isDirectory) {
				callback(false);
				return;
			}
			entry.removeRecursively(function(dirEntry) {
				callback(true);
				resetMetadata();
				saveMetadata();
			}, function(err) {
				callback(true);
				resetMetadata();
				saveMetadata();
			});
		}, function(err) {
			callback(false);
		});
	};
	
	// private function, get local file name by file url
	function getFileNameByUrl(url) {
		if (!url) {
			return '';
		}
		var idx = url.lastIndexOf('/');
		if (-1 == idx) {
			return url;
		}
		return url.substring(idx + 1);
	}
	
	// private function, get file path by file name
	function getFilePathByName(fileName) {
		if (!fileName || !checkContext()) {
			return '';
		}
		var len = metadata.keyArray.length;
		var keyTmp = '';
		for (var i=0; i < len; ++i) {
			keyTmp = metadata.keyArray[i];
			if (fileName == keyTmp) {
				return metadata.valueArray[i];
			}
		}
		return false;
	}
	
	// private function, add new item to storage
	function addToStorage(fileName, localFileUrl) {
		if (!fileName || !localFileUrl || !checkContext()) {
			return;
		}
		metadata.keyArray.push(fileName);
		metadata.valueArray.push(localFileUrl);
		saveMetadata();
	}
	
	// get local file path by url(include server domain)
	owner.localPathForUrl = function(url, callback, downloadIfNeed) {
		callback = callback || $.noop;
		if (!checkContext()) {
			callback('', url);
			return;
		}
		if (!url) {
			callback('', url);
			return;
		}
		if (0 == url.indexOf('file:///')) {
			callback(url, url);
			return;
		}
		var fileName = getFileNameByUrl(url);
		var filePath = getFilePathByName(fileName);
		if (!filePath) {
			downloadIfNeed = downloadIfNeed || true;
			if (!downloadIfNeed) {
				callback('', url);
				return;
			}
			// download file
			var downloadTask = downloader.createDownload(url, {method: 'GET', priority: 10, retry: 1,
												filename: '_downloads/' + fileName}, function(download, statusCode) {
				if (200 != statusCode) {
					callback('', url);
					return;
				}
				var localFileName = download.filename;
				if (!localFileName) {
					callback('', url);
					return;
				}
				plusObj.io.resolveLocalFileSystemURL(localFileName, function(entry) {
					var localFileUrl = entry.toLocalURL();
					callback(localFileUrl, url);
					addToStorage(fileName, localFileUrl);
				}, function(e) {
					callback('', url);
				});
			});
			downloadTask.start();
		} else {
			callback(filePath, url);
		}
	};
}(mui, document, window.fileManager = {}));
