(function($, owner) {
	/**
	 * 从相册选择图片。
	 * @param {Object} plusObj: Object, 全局plus对象。
	 * @param {Function} callback: Function, 回调函数，参数：filePath(图片路径)，errMsg(错误信息，成功时为空)。
	 */
	owner.selectImgFromGallery = function(plusObj, callback) {
		callback = callback || $.noop;
		plusObj.gallery.pick(function(path) {
			callback(path);
		}, function(err) {
			if (-2 != err.code) {
				callback(null, '出错，请重试');
			}
		});
	};
	
	/**
	 * 拍照。
	 * @param {Object} plusObj: Object, 全局plus对象。
	 * @param {Function} callback: Function, 回调函数，参数：filePath(图片路径)，errMsg(错误信息，成功时为空)。
	 */
	owner.selectImgFromCamera = function(plusObj, callback) {
		callback = callback || $.noop;
		if (!owner.camera) {
			owner.camera = plusObj.camera.getCamera() || {};
      		if (!owner.camera) {
      			callback(null, '无法打开拍照功能' + ('ios' == plusObj.os.name ? '，请在设置中允许使用相机' : ''));
      			return;
      		}
      	}
      	owner.camera.captureImage(function(path) {
      		plusObj.io.resolveLocalFileSystemURL(path, function(entry) {
      			callback(entry.toLocalURL());
      		}, function(err) {
      			callback(null, '出错，请重试');
      		});
      	}, function(err) {
      		if (2 != err.code) {
      			callback(null, '出错，请重试');
      		}
      	}, {format: 'jpg'});
	};
	
	/**
	 * 从本地选择图片文件。
	 * @param {Object} plusObj: Object, 全局plus对象
	 * @param {Function} callback: Function, 回调函数，参数：filePath(图片路径)，errMsg(错误信息，成功时为空)。
	 * @param {String} from: String, 图片来源，可选值：'gallery'(相册)，'camera'(拍照)，其它(让用户选择)。默认值空，可选。
	 */
	owner.selectImg = function(plusObj, callback, from) {
		callback = callback || $.noop;
		if ('gallery' == from) {
			owner.selectImgFromGallery(plusObj, callback);
			return;
		} else if ('camera' == from) {
			onwer.selectImgFromCamera(plusObj, callback);
			return;
		}
		plusObj.nativeUI.actionSheet({
			title: '选择图片来源',
			cancel: '取消',
			buttons: [{
				title: '相册'
			}, {
				title: '拍照'
			}]
		}, function(ev) {
			if (1 == ev.index) {
				owner.selectImgFromGallery(plusObj, callback);
			} else if (2 == ev.index) {
				owner.selectImgFromCamera(plusObj, callback);
			}
		});
	};
}(mui, window.imgSelector = {}));