(function(owner) {
	owner.server = commonProtocol.serverDomain + '/pb-upload-json';
	owner.tasks = new Array(); // 保存所有任务的数据
	//定义数据结构
	function UploadInfo(upload, path, scb, pcb) {
		this.uploader = upload || null;
		this.filePath = path || null;
		this.successCB = scb || null;
		this.progressCB = pcb || null;
	}
	// 遍历数组，查找相关数据
	owner.findUploadInfoByFilePath = function(path) {
		for (oneTask in owner.tasks) {
			if (path == oneTask.filePath) {
				return oneTask;
			}
		}
		return null;
	};
	// 删除一个UploadInfo
	owner.delUploadInfo = function(info) {
		var path = 'string' == typeof a ? info : info.filePath;
		var index = -1;
		for (var i = 0; i < owner.tasks.length; ++i) {
			if (path == owner.tasks[i].filePath) {
				index = i;
				break;
			}
		}
		if (-1 < index) {
			owner.tasks.splice(index, 1);
		}
	}
	// 更新上传进度timer回调
	window.updateProgress = function(filePath) {
		var task = owner.findUploadInfoByFilePath(filePath);
		if (!task) {
			return;
		}
		var pcb = task.progressCB;
		if (!pcb) {
			return;
		}
		var upload = task.uploader;
		if (!upload) {
			return;
		}
		var progress = Math.round((upload.uploadedSize || 0) / (upload.totalSize || 1) * 100.0);
		if (0 < progress) {
			pcb(progress);
		}
		if (100 > progress) {
			setTimeout('updateProgress("' + filePath + '");', 1500);
		}
	};
	/**
	 * 私有方法，开始上传文件。
	 * @param {Object} plusUploader
	 * @param {Object} filePath
	 * @param {Object} callback
	 * @param {Object} progressCallback
	 */
	function doStartUploadFile(plusUploader, filePath, callback, progressCallback) {
		var task = plusUploader.createUpload(owner.server, {method: 'POST', blocksize: 0}, function(upload, status) {
			if (200 != status) {
				var errMsg = '未知错误';
				if (400 <= status && 500 > status) {
					errMsg = '服务器地址错误';
				} else if (500 <= status) {
					errMsg = '服务器出错';
				}
				callback(filePath, '', 1, errMsg);
				return;
			}
			var resultData = upload.responseText;
			var key = resultData.substring(0, 11);
			if ('resultData:' == key) {
				resultData = resultData.substr(11);
			}
			try{
				resultData = JSON.parse(resultData);
			}catch(e){
				callback(filePath, '', 1, '解析数据失败');
				return;
			}
			
			callback(filePath, resultData.fileUrl || '', 0, '');
			owner.delUploadInfo(filePath);
		});
		task.addFile(filePath, {key: "file"});
		task.setRequestHeader('Accept', 'text/*');
		owner.tasks.push(new UploadInfo(task, filePath, callback, progressCallback));
		task.start();
		// start progress timer
		progressCallback && setTimeout('updateProgress("' + filePath + '");', 1500);
	}
	/**
	 * 上传单个文件函数，如果是图片文件则检查是否需要压缩。
	 * @param {Object} plusUploader: Object, 全局plus.uploader对象，要plus初始化完成后才可以使用。
	 * @param {String} filePath: String, 文件路径。
	 * @param {Function} callback: Function, 回调函数，其参数为：fileName(上传的文件), resultData(服务器返回数据), code(0-成功，1-出错，2-取消)，msg(出错信息)。可选。
	 * @param {Function} progressCallback: Function, 上传进度回调函数，其参数为：progress(数字1-100)。可选。
	 */
	owner.startUploadFile = function(plusUploader, filePath, callback, progressCallback) {
		var fileName = filePath.split('/').pop();
		var suffix = /\.[^\.]+$/.exec(fileName); 
//		console.log('file suffix: ' + suffix + '   , fileName: ' + fileName);
		if (plus && /png|PNG|jpg|JPG|jpeg|JPEG$/.test(filePath)) {
			var bitmap = new plus.nativeObj.Bitmap();
			bitmap.load(filePath, function() {
				bitmap.save(filePath.replace(fileName, '') + new Date().getTime() + suffix, {quality: 0.2}, function(event) {
					bitmap.clear();
					doStartUploadFile(plusUploader, event.target, callback, progressCallback);
				}, function(exp) {
					bitmap.clear();
					doStartUploadFile(plusUploader, filePath, callback, progressCallback);
				});
			}, function(exp) {
				bitmap.clear();
				doStartUploadFile(plusUploader, filePath, callback, progressCallback);
			});
		} else {
			doStartUploadFile(plusUploader, filePath, callback, progressCallback);
		}
	};
	/**
	 * 暂停一个上传任务。
	 * @param {String} filePath: String, 文件路径。
	 */
	owner.pauseTask = function(filePath) {
		var task = owner.findUploadInfoByFilePath(filePath);
		task && task.pause();
	};
	/**
	 * 恢复一个上传任务。
	 * @param {String} filePath: String, 文件路径。
	 */
	owner.resumeTask = function(filePath) {
		var task = owner.findUploadInfoByFilePath(filePath);
		task && task.resume();
	};
	/**
	 * 取消一个上传任务。
	 * @param {String} filePath: String, 文件路径。
	 * @param {Boolean} invokeCallback: Boolean, 是否调用上传完成回调函数。默认为false，可选。
	 */
	owner.abortTask = function(filePath, invokeCallback) {
		var task = owner.findUploadInfoByFilePath(filePath);
		if (!task) {
			return;
		}
		task.abort();
		var successCB = task.successCB;
		if (successCB) {
			if (invokeCallback) {
				successCB(filePath, '', 2, '取消上传');
			}
		}
		owner.delUploadInfo(filePath);
	};
}(window.fileUploader = {}));
